
' Demo code showing how to delete records from the database
' and then traverse through the database ignoring deleted
' records.

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[2]

'define the names of the database & index
  DBFname$ = "Cheetah.dbf"
  IDXname$ = "Cheetah.idx"

'define the structure of the databases
  Fd$[0] = "CUSTID,C,7,0"     'Character field, length 7
  Fd$[1] = "CUSTNAME,C,35,0"  'Character field, length 35
  Fd$[2] = "SALARY,N,14,2"    'Numeric field, length 14, 2 decimal places (i.e "###########.##")

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

' some languages do not recognize the string array that is passed to
' the xdbCreate function (e.g. UltiMade). In those cases you should
' use the following alternative database creation code.

' xdbAddField("CUSTID,C,7,0")
' xdbAddField("CUSTNAME,C,35,0")
' xdbAddField("SALARY,N,14,2)
' xdbCreateFields(DBFname$)   'this creates the database

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "UPPER(CUSTID)"   'index is not case sensitive
  Duplicates = $$XDBTRUE         'allow duplicate customer ID's

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'open the index
  idxHandle = xdbOpenIndex (IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError&) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
  NumRecs = 200

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.
  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")

	CustIDLength = xdbFieldLength(dbHandle, CustIDfield)

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer

		 CustID$ = RJUST$ (STRING$(x), CustIDLength)
     xdbAssignField(dbHandle, "", CustIDfield, @CustID$)
     xdbAssignField(dbHandle, "", CustNameField, @"The big Cat")
     xdbAssignField(dbHandle, "", SalaryField, @"-1234.50")

     'add to the end of the database (Append)  add the key to the index.
     xdbAddRecord(dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding record.")
        EXIT FOR
     END IF

  NEXT x


  ? "Database Records BEFORE delete: " + STR$(xdbRecordCount(dbHandle))

'delete 3 records. The record must first be loaded into the record buffer
'before it can be marked for deletion. Records are not physically removed
'from the database until xdbPack or xdbZap is called. This gives you the
'opportunity to recall (unmark from deletion) records using the xdbRecall
'function call.

  xdbDeleteRecord(dbHandle, 100)    'mark record 100 for deletion
  xdbDeleteRecord(dbHandle, 101)    'mark record 101 for deletion
  xdbDeleteRecord(dbHandle, 102)    'mark record 102 for deletion


'notice the number of records have not changed. The records are only marked
'for deletion. IF you are moving through the database and want to exclude
'processing any deleted records then you should test the record using the
'xdbDeleted function or set the xdbSkipDeleted function to %XDBTRUE.
  ? "Database Records AFTER delete: " + STR$(xdbRecordCount(dbHandle))

'now move through the database ignoring all deleted records (using the
'idxHandle index)
  ? "Move through records ignoring deleted records (xdbDeleted)."

  xdbMoveFirst(dbHandle, idxHandle)

  NumProcessed = 0
  DO UNTIL xdbEOF(dbHandle)

     IF xdbDeleted(dbHandle, xdbRecordNumber(dbHandle)) THEN
        'this record is deleted so skip it!
     ELSE
        'process the record
        NumProcessed = NumProcessed + 1
     END IF

     xdbMoveNext(dbHandle, idxHandle)

  LOOP

  ? "Processed"  STR$(NumProcessed) + " non-deleted records in the database."

'now move through the database ignoring all deleted records (using the
'idxHandle index). Use the alternate method of using xdbSkipDeleted.
  ? "Move through records ignoring deleted records (xdbSkipDeleted)."

  xdbSkipDeleted (dbHandle, $$XDBTRUE)

  xdbMoveFirst(dbHandle, idxHandle)

  NumProcessed = 0
  DO UNTIL xdbEOF(dbHandle)

     'process the record
     NumProcessed = NumProcessed + 1

     xdbMoveNext(dbHandle, idxHandle)

  LOOP

  ? "Processed"  STR$(NumProcessed) + " non-deleted records in the database."

'pack the database. This will physically remove all records marked for
'deletion. The changes are permanent - records can not be recalled after
'this function is called. Any open indexes are automatically reindexed.

  xdbPack(dbHandle)

'the record count should now be properly reflected.
  ? "Database Records AFTER Pack: " +  STR$(xdbRecordCount(dbHandle))

'zap the database. This instantly deletes all records and related index
'keys. You can not recover from this process so it is advisable to make
'backups of the database and indexes prior to calling this function. You
'do not have to manually call the idxReindex routine.

  xdbZap(dbHandle)

'the record count should now be zero.
  ? "Database Records AFTER zap: " + STR$(xdbRecordCount(dbHandle))


'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM