
' Demo code showing the creation of a database that uses
' Cheetah extended data types.
'
'WARNING: Using extended data types will deviate the database from
'         the standard xbase database format therefore other programs
'         expecting an xbase file will not read the database correctly.
'         However, using the extended types will make your databases
'         smaller and more efficient if you can afford to forego the
'         compatibility issue.
'
'         ** Also, you can not index based on extended data types unless
'            you use the xdbMKL or xdbMKI functions. **

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[7]

'define the names of the database & index
  DBFname$ = "Cheetah.dbf"
  IDXname$ = "Cheetah.idx"

'define the structure of the databases
  Fd$[0] = "CUSTID,C,7,0"     'Character field, length 7
  Fd$[1] = "CUSTNAME,C,35,0"  'Character field, length 35
  Fd$[2] = "NUM1,I,2,0"      'Integer field, length 2 (will always be 2 regardless of the number entered)
  Fd$[3] = "NUM2,W,4,0"      'Long field, length 4 (will always be 4 regardless of the number entered)
  Fd$[4] = "NUM3,S,4,0"      'Single field, length 4 (will always be 4 regardless of the number entered)
  Fd$[5] = "NUM4,X,8,0"      'Double field, length 8 (will always be 8 regardless of the number entered)
  Fd$[6] = "NUM5,Y,8,0"      'Currency field, length 8 (will always be 8 regardless of the number entered)
  Fd$[7] = "NOTES,Z,4,0"     'Binary memo field, length always 4. Replaces the "M" datatype which is length 10.

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "UPPER(CUSTID)"   'index is not case sensitive
  Duplicates = $$XDBTRUE         'allow duplicate customer ID's

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'open the index
  idxHandle = xdbOpenIndex (IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError&) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
  NumRecs = 200

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.
  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  IntegerField = xdbFieldNumber(dbHandle, "NUM1")
  LongField = xdbFieldNumber(dbHandle, "NUM2")
  SingleField = xdbFieldNumber(dbHandle, "NUM3")
  DoubleField = xdbFieldNumber(dbHandle, "NUM4")
  CurrencyField = xdbFieldNumber(dbHandle, "NUM5")

  CustIDLength = xdbFieldLength(dbHandle, CustIDfield)

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer

     CustID$ = RJUST$ (STRING$(x), CustIDLength)
     xdbAssignField(dbHandle, "", CustIDfield, @CustID$)
     xdbAssignField(dbHandle, "", CustNameField, @"The big Cat")
     xdbAssignFieldINT(dbHandle, "", IntegerField, 322)
     xdbAssignFieldLNG(dbHandle, "", LongField, 567322)
     xdbAssignFieldSNG(dbHandle, "", SingleField, 1234.5433)
     xdbAssignFieldDBL(dbHandle, "", DoubleField, 546677.5433439)
     xdbAssignFieldCUR(dbHandle, "", CurrencyField, 567.45)

     'add to the end of the database (Append)  add the key to the index.
     xdbAddRecord(dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding record.")
        EXIT FOR
     END IF

  NEXT x

	? "Database and Index created."
  ? "Database Records : " + STR$(xdbRecordCount(dbHandle))

'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM