
' Demo code showing how to add and delete memo records. Memo
' fields are created using the "M" or "Z" field types. The
' actual memo data is stored in a separate file. For this
' reason it is more efficient for you to store smaller strings
' in "C" character fields. Those fields can store strings up
' to 256 characters in length. Any strings longer than that
' will have to be stored in memo fields.

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[3]

'define the names of the database & index
  DBFname$ = "Cheetah.dbf"
  IDXname$ = "Cheetah.idx"

'define the structure of the databases
  Fd$[0] = "CUSTID,C,7,0"     'Character field, length 7
  Fd$[1] = "CUSTNAME,C,35,0"  'Character field, length 35
  Fd$[2] = "SALARY,N,14,2"    'Numeric field, length 14, 2 decimal places (i.e "###########.##")
	Fd$[3] = "NOTES,M,10,0"     'Memo field, this is a pointer into the .dbt memo file

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "UPPER(CUSTID)"   'index is not case sensitive
  Duplicates = $$XDBTRUE         'allow duplicate customer ID's

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'open the index
  idxHandle = xdbOpenIndex (IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError&) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
'NOTE: When adding a large number of records it is better to close
'the index, do the appends to the database and then open the
'index and reindex. For a few hundred records there should be no
'performance issues.

  NumRecs = 200

	XstAlert ( "Press 'Okay' to create " + STR$(NumRecs) + " database and index records.")

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.

  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")
	NotesField = xdbFieldNumber(dbHandle, "NOTES")

  CustIDLength = xdbFieldLength(dbHandle, CustIDfield)

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer

     CustID$ = RJUST$ (STRING$(x), CustIDLength)
     xdbAssignField(dbHandle, "", CustIDfield, @CustID$)
     xdbAssignField(dbHandle, "", CustNameField, @"The big Cat")
     xdbAssignField(dbHandle, "", SalaryField, "-1234.50")

     'add to the end of the database (Append)  add the key to the index.
     xdbAddRecord(dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding record.")
        EXIT FOR
     END IF

  NEXT x

	? "Database and Index records created."
  ? "Database Records : " + STR$(xdbRecordCount(dbHandle))

	'update record #100 by adding a notes memo to it.
  Notes$ = "This is my first note that I am adding to record #100."
  xdbGetRecord(dbHandle, 100)
  xdbAssignField(dbHandle, "", NotesField, Notes$)
  xdbPutRecord(dbHandle, 100)

'display the note
  xdbGetRecord(dbHandle, 100)
  Notes$ = xdbFieldValue$(dbHandle, "", NotesField)

  ? "Memo for Record# 100: " + Notes$

'delete the note (this is accomplished by writing "" to the memo field.
  xdbGetRecord(dbHandle, 100)
  xdbAssignField(dbHandle, "", NotesField, "")
  xdbPutRecord(dbHandle, 100)

'display the note (it should not exist)
  xdbGetRecord(dbHandle, 100)
  Notes$ = xdbFieldValue$(dbHandle, "", NotesField)

  ? "Updated Memo (deleted) for Record# 100: " + Notes$


'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM