
' Demo code showing how to reindex databases.

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"
	IMPORT "kernel32"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[2]

'define the names of the database & index
  DBFname$ = "Cheetah.dbf"
  IDXname$ = "Cheetah.idx"

'define the structure of the databases
  Fd$[0] = "CUSTID,C,7,0"     'Character field, length 7
  Fd$[1] = "CUSTNAME,C,35,0"  'Character field, length 35
  Fd$[2] = "SALARY,N,14,2"    'Numeric field, length 14, 2 decimal places (i.e "###########.##")

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "UPPER(CUSTID)"   'index is not case sensitive
  Duplicates = $$XDBFALSE

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
'NOTE: When adding a large number of records it is better to close
'the index, do the appends to the database and then open the
'index and reindex. For a few hundred records there should be no
'performance issues.

  NumRecs = 5000

	XstAlert ( "Press 'Okay' to create " + STR$(NumRecs) + " database and index records.")

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.

  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")

  CustIDLength = xdbFieldLength(dbHandle, CustIDfield)

  'turn on the SpeedAppend buffer
  xdbSpeedAppend(dbHandle, $$XDBTRUE)

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer
		 CustID$ = RJUST$ (STRING$(x+1000), CustIDLength)
     xdbAssignField(dbHandle, "", CustIDfield, @CustID$)
     xdbAssignField(dbHandle, "", CustNameField, @"The big Cat")
     xdbAssignField(dbHandle, "", SalaryField, STR$(1000 + x))

     'add to the end of the database (Append)
		 'this does not update the index.
     xdbAppendRecord(dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding database record.")
        EXIT FOR
     END IF

  NEXT x

	? "Database and Index records created."
  ? "Database Records : " + STR$(xdbRecordCount(dbHandle))

  'turn off the SpeedAppend buffer to ensure all records are written
  xdbSpeedAppend(dbHandle, $$XDBFALSE)

'open the index
  idxHandle = xdbOpenIndex(IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ( "Error: " + STR$(xdbError()) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'create the index by reindexing
	XstAlert ("Press OK to start Reindex...")

	startTime = GetTickCount ()

  xdbReindexAll(dbHandle)

	endTime = GetTickCount ()

	? "Reindex " + STR$(NumRecs) + " records: " + FORMAT$("####", (endTime-startTime)) + " ms."

	IF xdbError() THEN
		 XstAlert ("Error: " + STR$(xdbError()))
	END IF

	? "Record Count:" + STR$(xdbRecordCount(dbHandle)) + "  Key Count:" + STR$(xdbKeyCount(dbHandle,idxHandle))


'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM