
' Demo code showing performance of Cheetah's xdbSpeedAppend.

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"
	IMPORT "kernel32"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[0]

'define the names of the database & index
  DBFname$ = "Speed.dbf"
  IDXname$ = "Speed.idx"

	? "This is a simple demo that creates a 10,000 record sample database and index."
	? "It demonstrates Cheetah's speed at creating databases from scratch."

'define the structure of the databases
  Fd$[0] = "POSTAL,C,5,0"    'Character field, length 5

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "POSTAL"   				'index is not case sensitive
  Duplicates = $$XDBTRUE         	'allow duplicate customer ID's

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'open the index
  idxHandle = xdbOpenIndex (IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError&) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
'NOTE: When adding a large number of records it is better to close
'the index, do the appends to the database and then open the
'index and reindex. For a few hundred records there should be no
'performance issues.

  NumRecs = 10000

	SpeedOn = $$XDBTRUE
'  SpeedOn = $$XDBFALSE

	XstAlert ("Press 'Okay' to create " + STR$(NumRecs) + " database records.")

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.

	PostalField = xdbFieldNumber(dbHandle, "POSTAL")

	PostalLength = xdbFieldLength(dbHandle, PostalField)

	IF SpeedOn = $$XDBTRUE THEN
  'turn on the buffered append writes
		xdbSpeedAppend(dbHandle, $$XDBTRUE)
	END IF

	StartTime = GetTickCount ()

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer

		 Postal$ = RJUST$ (STRING$(x+1000), PostalLength)

     xdbAssignField(dbHandle, "", PostalField, Postal$)

     'add to the end of the database (Append) + add the key to the index.
     xdbAppendRecord (dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding database record.")
        EXIT FOR
     END IF

  NEXT x

	IF SpeedOn = $$XDBTRUE THEN
		'make sure to turn off buffered append writes
		'so all records are written.
		xdbSpeedAppend (dbHandle, $$XDBFALSE)
	END IF


	StatusTime = GetTickCount ()
	TimeResults = StatusTime - StartTime

	IF xdbError() THEN
   XstAlert ("Error: " + STR$(xdbError()))
	ELSE
	  ? "Database Records Count       :" + STR$(xdbRecordCount(dbHandle))
    ? "Database Records Created.  ms:" + STR$(TimeResults) + " - RecsPerSec: " + STR$(INT(NumRecs/TimeResults*1000))
	END IF


'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM